@@ -86,6 +86,8 @@ pub struct Settings {
     pub interface_view: InterfaceView,
     pub result_sort: ResultSort,
     pub disable_menu: bool,
+    pub prompt: String,
+    pub disable_run_command: bool,
 }
 
 impl Default for Settings {
@@ -115,6 +117,8 @@ impl Default for Settings {
             interface_view: InterfaceView::Top,
             result_sort: ResultSort::Rank,
             disable_menu: false,
+            prompt: String::from("$"),
+            disable_run_command: false,
         }
     }
 }
@@ -128,7 +132,7 @@ impl Settings {
             ..Default::default()
         };
 
-        settings.debug = cli.debug || env::var("MCFLY_DEBUG").is_ok();
+        settings.debug = cli.debug || is_env_var_truthy("MCFLY_DEBUG");
         settings.limit = env::var("MCFLY_HISTORY_LIMIT")
             .ok()
             .and_then(|o| o.parse::<i64>().ok());
@@ -280,7 +284,7 @@ impl Settings {
                 }
 
                 settings.delete_without_confirm =
-                    delete_without_confirm || env::var("MCFLY_DELETE_WITHOUT_CONFIRM").is_ok();
+                    delete_without_confirm || is_env_var_truthy("MCFLY_DELETE_WITHOUT_CONFIRM");
 
                 settings.output_selection = output_selection;
 
@@ -331,15 +335,23 @@ impl Settings {
             }
         }
 
-        settings.lightmode = env::var_os("MCFLY_LIGHT").is_some();
+        settings.lightmode = is_env_var_truthy("MCFLY_LIGHT");
 
-        settings.disable_menu = env::var_os("MCFLY_DISABLE_MENU").is_some();
+        settings.disable_menu = is_env_var_truthy("MCFLY_DISABLE_MENU");
+
+        settings.disable_run_command = is_env_var_truthy("MCFLY_DISABLE_RUN_COMMAND");
 
         settings.key_scheme = match env::var("MCFLY_KEY_SCHEME").as_ref().map(String::as_ref) {
             Ok("vim") => KeyScheme::Vim,
             _ => KeyScheme::Emacs,
         };
 
+        if let Ok(prompt) = env::var("MCFLY_PROMPT") {
+            if prompt.chars().count() == 1 {
+                settings.prompt = prompt;
+            }
+        }
+
         settings
     }
 
@@ -383,3 +395,17 @@ fn pwd() -> String {
         )
     })
 }
+
+fn is_env_var_truthy(name: &str) -> bool {
+    match env::var(name) {
+        Ok(val) => {
+            val != "F"
+                && val != "f"
+                && val != "false"
+                && val != "False"
+                && val != "FALSE"
+                && val != "0"
+        }
+        Err(_) => false,
+    }
+}
